fn unpackUnorm16X(packed: u32) -> f32 {
    return f32(packed & 0xFFFFu) / 65535.0;
}
fn unpackUnorm16Y(packed: u32) -> f32 {
    return f32((packed >> 16) & 0xFFFFu) / 65535.0;
}
fn unpackUint16X(packed: u32) -> u16 {
    return u16(packed & 0xFFFFu);
}
fn unpackUint16Y(packed: u32) -> u16 {
    return u16((packed >> 16) & 0xFFFFu);
}
fn unpackUint2x16(packed: u32) -> vec2<u32> {
    let x: u16 = u16(packed & 0xFFFFu);
    let y: u16 = u16((packed >> 16) & 0xFFFFu);
    return vec2<u32>(x, y);
}

#ifdef EMULATE_PACKING

/**
 * Unpack an encoded half float into a float.
 *
 * Taken from the Angle codebase.
 */
float unpackHalf(uint val) {
    uint sign = (val & 0x8000u) << 16;
    int exponent = int((val & 0x7C00u) >> 10);
    uint mantissa = val & 0x03FFu;
    float f32 = 0.0;
    if(exponent == 0)
    {
        if (mantissa != 0u)
        {
            const float scale = 1.0 / (1 << 24);
            f32 = scale * mantissa;
        }
    }
    else if (exponent == 31)
    {
        return bitcast<f32>(sign | 0x7F800000u | mantissa);
    }
    else
    {
        exponent -= 15;
        float scale;
        if(exponent < 0)
        {
            // The negative unary operator is buggy on OSX.
            // Work around this by using abs instead.
            scale = 1.0 / (1 << abs(exponent));
        }
        else
        {
            scale = 1 << exponent;
        }
        float decimal = 1.0 + float(mantissa) / float(1 << 10);
        f32 = scale * decimal;
    }
    if (sign != 0u)
    {
        f32 = -f32;
    }
    return f32;
}

mediump vec2 unpackHalf2x16(highp uint val) {
    return vec2(unpackHalf(val & 0xFFFFu), unpackHalf(val >> 16u));
}

#endif

fn unpackUnormVector(v: vec2<u32>) -> vec4<f16> {
    return vec4<f16>(
        unpack2x16unorm(v.x),
        unpack2x16unorm(v.y)
    );
}

fn unpackSnormVector(v: vec2<u32>) -> vec4<f16> {
    return vec4<f16>(
        unpack2x16snorm(v.x),
        unpack2x16snorm(v.y)
    );
}

fn packUnormVector(v: vec4<f16>) -> vec2<u32> {
    return vec2<u32>(
        pack2x16unorm(v.xy),
        pack2x16unorm(v.zw)
    );
}

fn packSnormVector(v: vec4<f16>) -> vec2<u32> {
    return vec2<u32>(
        pack2x16snorm(v.xy),
        pack2x16snorm(v.zw)
    );
}
